@@ -41,7 +41,7 @@
    $make -j32
 
  * Build vp9 fuzzer
-   $ $CXX $CXXFLAGS -std=gnu++11 -DDECODER=vp9 \
+   $ $CXX $CXXFLAGS -std=gnu++17 -DDECODER=vp9 \
    -fsanitize=fuzzer -I../libvpx -I. -Wl,--start-group \
    ../libvpx/examples/vpx_dec_fuzzer.cc -o ./vpx_dec_fuzzer_vp9 \
    ./libvpx.a -Wl,--end-group
@@ -69,6 +69,7 @@
 #include <algorithm>
 #include <memory>
 
+#include "third_party/nalloc/nalloc.h"
 #include "vpx/vp8dx.h"
 #include "vpx/vpx_decoder.h"
 #include "vpx_ports/mem_ops.h"
@@ -85,33 +86,64 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
   if (size <= IVF_FILE_HDR_SZ) {
     return 0;
   }
+  nalloc_init(nullptr);
 
   vpx_codec_ctx_t codec;
   // Set thread count in the range [1, 64].
   const unsigned int threads = (data[IVF_FILE_HDR_SZ] & 0x3f) + 1;
   vpx_codec_dec_cfg_t cfg = { threads, 0, 0 };
-  if (vpx_codec_dec_init(&codec, VPXD_INTERFACE(DECODER), &cfg, 0)) {
+  vpx_codec_flags_t flags = 0;
+  if ((data[IVF_FILE_HDR_SZ] & 0x40) != 0) {
+    flags |= VPX_CODEC_USE_POSTPROC;
+  }
+  vpx_codec_err_t err =
+      vpx_codec_dec_init(&codec, VPXD_INTERFACE(DECODER), &cfg, flags);
+  if (err == VPX_CODEC_INCAPABLE) {
+    // vpx_codec_dec_init may fail with VPX_CODEC_USE_POSTPROC
+    // if the library is configured with --disable-postproc.
+    flags = 0;
+    if (vpx_codec_dec_init(&codec, VPXD_INTERFACE(DECODER), &cfg, flags)) {
+      return 0;
+    }
+  } else if (err != 0) {
     return 0;
   }
 
+  nalloc_start(data, size);
+
   if (threads > 1) {
     const int enable = (data[IVF_FILE_HDR_SZ] & 0xa0) != 0;
-    const vpx_codec_err_t err =
-        vpx_codec_control(&codec, VP9D_SET_LOOP_FILTER_OPT, enable);
-    static_cast<void>(err);
+    err = vpx_codec_control(&codec, VP9D_SET_LOOP_FILTER_OPT, enable);
   }
 
   data += IVF_FILE_HDR_SZ;
   size -= IVF_FILE_HDR_SZ;
 
+  int frame_cnt = 0;
   while (size > IVF_FRAME_HDR_SZ) {
     size_t frame_size = mem_get_le32(data);
     size -= IVF_FRAME_HDR_SZ;
     data += IVF_FRAME_HDR_SZ;
     frame_size = std::min(size, frame_size);
 
-    const vpx_codec_err_t err =
-        vpx_codec_decode(&codec, data, frame_size, nullptr, 0);
+    vpx_codec_stream_info_t stream_info;
+    stream_info.sz = sizeof(stream_info);
+    err = vpx_codec_peek_stream_info(VPXD_INTERFACE(DECODER), data, size,
+                                     &stream_info);
+
+    ++frame_cnt;
+    if (flags & VPX_CODEC_USE_POSTPROC) {
+      if (frame_cnt % 16 == 4) {
+        vp8_postproc_cfg_t pp = { 0, 0, 0 };
+        if (vpx_codec_control(&codec, VP8_SET_POSTPROC, &pp)) goto fail;
+      } else if (frame_cnt % 16 == 12) {
+        vp8_postproc_cfg_t pp = { VP8_DEBLOCK | VP8_DEMACROBLOCK | VP8_MFQE, 4,
+                                  0 };
+        if (vpx_codec_control(&codec, VP8_SET_POSTPROC, &pp)) goto fail;
+      }
+    }
+
+    err = vpx_codec_decode(&codec, data, frame_size, nullptr, 0);
     static_cast<void>(err);
     vpx_codec_iter_t iter = nullptr;
     vpx_image_t *img = nullptr;
@@ -120,6 +152,8 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
     data += frame_size;
     size -= frame_size;
   }
+fail:
   vpx_codec_destroy(&codec);
+  nalloc_end();
   return 0;
 }
