@@ -19,25 +19,25 @@
 namespace {
 
 void reset(struct WebmInputContext *const webm_ctx) {
-  if (webm_ctx->reader != NULL) {
+  if (webm_ctx->reader != nullptr) {
     mkvparser::MkvReader *const reader =
         reinterpret_cast<mkvparser::MkvReader *>(webm_ctx->reader);
     delete reader;
   }
-  if (webm_ctx->segment != NULL) {
+  if (webm_ctx->segment != nullptr) {
     mkvparser::Segment *const segment =
         reinterpret_cast<mkvparser::Segment *>(webm_ctx->segment);
     delete segment;
   }
-  if (webm_ctx->buffer != NULL) {
+  if (webm_ctx->buffer != nullptr) {
     delete[] webm_ctx->buffer;
   }
-  webm_ctx->reader = NULL;
-  webm_ctx->segment = NULL;
-  webm_ctx->buffer = NULL;
-  webm_ctx->cluster = NULL;
-  webm_ctx->block_entry = NULL;
-  webm_ctx->block = NULL;
+  webm_ctx->reader = nullptr;
+  webm_ctx->segment = nullptr;
+  webm_ctx->buffer = nullptr;
+  webm_ctx->cluster = nullptr;
+  webm_ctx->block_entry = nullptr;
+  webm_ctx->block = nullptr;
   webm_ctx->block_frame_index = 0;
   webm_ctx->video_track_index = 0;
   webm_ctx->timestamp_ns = 0;
@@ -84,7 +84,7 @@ int file_is_webm(struct WebmInputContext *webm_ctx,
   }
 
   const mkvparser::Tracks *const tracks = segment->GetTracks();
-  const mkvparser::VideoTrack *video_track = NULL;
+  const mkvparser::VideoTrack *video_track = nullptr;
   for (unsigned long i = 0; i < tracks->GetTracksCount(); ++i) {
     const mkvparser::Track *const track = tracks->GetTrackByIndex(i);
     if (track->GetType() == mkvparser::Track::kVideo) {
@@ -94,7 +94,7 @@ int file_is_webm(struct WebmInputContext *webm_ctx,
     }
   }
 
-  if (video_track == NULL || video_track->GetCodecId() == NULL) {
+  if (video_track == nullptr || video_track->GetCodecId() == nullptr) {
     rewind_and_reset(webm_ctx, vpx_ctx);
     return 0;
   }
@@ -137,12 +137,12 @@ int webm_read_frame(struct WebmInputContext *webm_ctx, uint8_t **buffer,
   do {
     long status = 0;
     bool get_new_block = false;
-    if (block_entry == NULL && !block_entry_eos) {
+    if (block_entry == nullptr && !block_entry_eos) {
       status = cluster->GetFirst(block_entry);
       get_new_block = true;
     } else if (block_entry_eos || block_entry->EOS()) {
       cluster = segment->GetNext(cluster);
-      if (cluster == NULL || cluster->EOS()) {
+      if (cluster == nullptr || cluster->EOS()) {
         *buffer_size = 0;
         webm_ctx->reached_eos = 1;
         return 1;
@@ -150,22 +150,22 @@ int webm_read_frame(struct WebmInputContext *webm_ctx, uint8_t **buffer,
       status = cluster->GetFirst(block_entry);
       block_entry_eos = false;
       get_new_block = true;
-    } else if (block == NULL ||
+    } else if (block == nullptr ||
                webm_ctx->block_frame_index == block->GetFrameCount() ||
                block->GetTrackNumber() != webm_ctx->video_track_index) {
       status = cluster->GetNext(block_entry, block_entry);
-      if (block_entry == NULL || block_entry->EOS()) {
+      if (block_entry == nullptr || block_entry->EOS()) {
         block_entry_eos = true;
         continue;
       }
       get_new_block = true;
     }
-    if (status || block_entry == NULL) {
+    if (status || block_entry == nullptr) {
       return -1;
     }
     if (get_new_block) {
       block = block_entry->GetBlock();
-      if (block == NULL) return -1;
+      if (block == nullptr) return -1;
       webm_ctx->block_frame_index = 0;
     }
   } while (block_entry_eos ||
@@ -179,9 +179,9 @@ int webm_read_frame(struct WebmInputContext *webm_ctx, uint8_t **buffer,
       block->GetFrame(webm_ctx->block_frame_index);
   ++webm_ctx->block_frame_index;
   if (frame.len > static_cast<long>(*buffer_size)) {
-    delete[] * buffer;
+    delete[] *buffer;
     *buffer = new uint8_t[frame.len];
-    if (*buffer == NULL) {
+    if (*buffer == nullptr) {
       return -1;
     }
     webm_ctx->buffer = *buffer;
@@ -198,7 +198,7 @@ int webm_read_frame(struct WebmInputContext *webm_ctx, uint8_t **buffer,
 int webm_guess_framerate(struct WebmInputContext *webm_ctx,
                          struct VpxInputContext *vpx_ctx) {
   uint32_t i = 0;
-  uint8_t *buffer = NULL;
+  uint8_t *buffer = nullptr;
   size_t buffer_size = 0;
   while (webm_ctx->timestamp_ns < 1000000000 && i < 50) {
     if (webm_read_frame(webm_ctx, &buffer, &buffer_size)) {
@@ -210,10 +210,12 @@ int webm_guess_framerate(struct WebmInputContext *webm_ctx,
   vpx_ctx->framerate.denominator =
       static_cast<int>(webm_ctx->timestamp_ns / 1000);
   delete[] buffer;
+  // webm_ctx->buffer is assigned to the buffer pointer in webm_read_frame().
+  webm_ctx->buffer = nullptr;
 
   get_first_cluster(webm_ctx);
-  webm_ctx->block = NULL;
-  webm_ctx->block_entry = NULL;
+  webm_ctx->block = nullptr;
+  webm_ctx->block_entry = nullptr;
   webm_ctx->block_frame_index = 0;
   webm_ctx->timestamp_ns = 0;
   webm_ctx->reached_eos = 0;
